use rs2sr::config::*;
use rs2sr::radiosure::*;
use rs2sr::screamerradio::defaultdatas::*;
use rs2sr::screamerradio::userdatas::*;
use rs2sr::converter::*;
use std::env;
use std::process::ExitCode;

const ERRCODE_CFGERR: u8 = 1;
const ERRCODE_DEFSTN: u8 = 2;
const ERRCODE_RSDATA: u8 = 3;
const ERRCODE_SRDATA: u8 = 4;

fn main() -> ExitCode {
    // コマンドラインからconfigを作成
    let args: Vec<String> = env::args().collect();
    let cfg = match Config::new(args) {
        Ok(cfg) => cfg,
        Err(msg) => {
            if Config::is_no_error(&msg) {
                return ExitCode::SUCCESS;
            } else {
                eprintln!("{}", msg);
                return ERRCODE_CFGERR.into();
            }
        }
    };

    // Screamer Radioのdefault stationの局名を取得して、チェッカーを作成
    let srcname = cfg.get_stations_filename();
    let dsc = match DefaultStationChecker::new(srcname) {
        Ok(dsc) => dsc,
        Err(e) => {
            eprintln!("cannot get station names : {}", e);
            return ERRCODE_DEFSTN.into();
        }
    };

    // データファイルからradio局のデータを作成
    let rsdatas = convert_file_to_rsdatas(&cfg, dsc);
    let rsdatas = match rsdatas {
        Ok(rsdatas) => rsdatas,
        Err(msg) => {
            println!("something bad...{}", msg);
            return ERRCODE_RSDATA.into();
        }
    };

    // Radio Sure!のデータをScreamer Radioのデータに変換
    let srdatas = rsdatas_to_srdatas(rsdatas);

    // Screamer Radioのデータを書き出す
    let screamerradiosource = cfg.get_screamer_filename();
    match write_to_screamer_radio_data(&srdatas, screamerradiosource) {
        Ok(_) => println!("done."),
        Err(msg) => {
            eprintln!("cannot convert : {}", msg);
            return ERRCODE_SRDATA.into();
        },
    }

    // 終了！
    ExitCode::SUCCESS
}
