use rs2sr::config::*;
use rs2sr::screamerradio::defaultdatas::*;
use rs2sr::screamerradio::userdatas::*;
use rs2sr::radiosure::*;
use rs2sr::converter::*;
use std::fs;
use std::fs::File;
use std::io::{BufReader, ErrorKind};
use std::path::Path;

fn load_srdatas(srcname: &str) -> Result<Vec<Srdata>, String> {
    let path = Path::new(srcname);
    if !path.is_file() {
        return Err("File not found".to_string());
    }
    let file = match File::open(&path) {
        Err(err) => {
            return Err(format!("{}", err));
        }
        Ok(file) => file,
    };
    let reader = BufReader::new(file);
    match serde_json::from_reader::<BufReader<File>, Vec<Srdata>>(reader) {
        Ok(datas) => Ok(datas),
        Err(err) => Err(format!("{}", err)),
    }
}

#[test]
fn it_full() {
    let args: Vec<String> = vec![
        "dummy".to_string(),
        "-ge=Talk,News,Public".to_string(),
        "-ne=Russian".to_string(),
        "-df=tests/stations-sample.json".to_string(),
        "-rs=tests/stations-sample.rsd".to_string(),
        "-ss=tests/MyStations-sample.json".to_string(),
    ];

    let cfg = match Config::new(args) {
        Ok(cfg) => cfg,
        Err(_) => {
            assert!(false);
            return;
        }
    };

    let orgname = "tests/MyStations-sample-org.json";

    match fs::remove_file(cfg.get_screamer_filename()) {
        Ok(_) => {}
        Err(err) => assert_eq!(err.kind(), ErrorKind::NotFound),
    };

    let dsc = match DefaultStationChecker::new(cfg.get_stations_filename()) {
        Ok(dsc) => dsc,
        Err(e) => {
            assert!(false,"{}",e);
            return ;
        },
    };

    let rsdatas = match convert_file_to_rsdatas(&cfg, dsc) {
        Ok(datas) => datas,
        Err(msg) => {
            assert!(false, "{}", msg);
            return;
        }
    };

    let srdatas = rsdatas_to_srdatas(rsdatas);

    match write_to_screamer_radio_data(&srdatas, cfg.get_screamer_filename()) {
        Ok(_) => {
            let l_datas = match load_srdatas(cfg.get_screamer_filename()) {
                Ok(datas) => datas,
                Err(err) => {
                    assert!(false, "{}", err);
                    return;
                }
            };
            let r_datas = match load_srdatas(orgname) {
                Ok(datas) => datas,
                Err(err) => {
                    assert!(false, "{}", err);
                    return;
                }
            };
            assert_eq!(l_datas.len(), r_datas.len());
            let mut r_iter = r_datas.iter();
            for l in l_datas {
                let r = r_iter.next().unwrap();
                assert!(l.loosy_eq(&r), "{:#?}/{:#?}", l, r);
            }
        }
        Err(msg) => assert!(false, "{}", msg),
    };
}
